local _Questic = _Questic_ADDON_Globals;
local Tooltip = Questic_Widgets_Tooltip;
local Widgets = _QuesticWidgets
local CreateFrame, Minimap, atan2, cos, sin, GetCursorPosition, insert, pairs, UIParent = CreateFrame, Minimap, atan2, cos, sin, GetCursorPosition, table.insert, pairs, UIParent;

local MinimapIcon = CreateFrame("Button", nil, Minimap);
MinimapIcon:SetSize(32, 32);
MinimapIcon:SetFrameStrata("MEDIUM");
MinimapIcon:SetPoint("CENTER", -12, -80);
MinimapIcon:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight");
MinimapIcon:RegisterForDrag("LeftButton");
MinimapIcon.Backdrop = MinimapIcon:CreateTexture(nil, "BACKGROUND");
MinimapIcon.Backdrop:SetAllPoints();
MinimapIcon.Backdrop:SetTexture("Interface\\PETBATTLES\\BattleBar-AbilityBadge-Neutral")
MinimapIcon.Icon = MinimapIcon:CreateTexture(nil, "BORDER");
MinimapIcon.Icon:SetSize(16, 16);
MinimapIcon.Icon:SetPoint("CENTER");
MinimapIcon.Icon:SetTexture("Interface\\AddOns\\Questic\\Media\\ActiveQuestIconMask.blp")
MinimapIcon.Icon:SetVertexColor(1,0.82,0);
MinimapIcon.Border = MinimapIcon:CreateTexture(nil, "BORDER");
MinimapIcon.Border:SetAllPoints();
MinimapIcon.Border:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder");
MinimapIcon.Border:SetTexCoord(-0.025, 0.625, -0.05, 0.6);
MinimapIcon.Border:SetRotation(-1.57079632679);

MinimapIcon:SetScript("OnEnter", function(self)
    Tooltip:SetTT(self,"Questic\n|cFFFFFFFFLeft-Click: Quick share\nRight-Click: Quick options",-8,-8);
end)


MinimapIcon:SetScript("OnLeave", function()
    Tooltip:Hide();
end)

MinimapIcon:SetScript("OnDragStart", function(self)
    if not self.isLocked then
        self:SetScript("OnUpdate", function(self)
            local MapX, MapY = Minimap:GetCenter();
            local MouseX, MouseY = GetCursorPosition();
            local MapScale = Minimap:GetEffectiveScale();
            MapX = MapX * MapScale;
            MapY = MapY * MapScale;
            local Ang = atan2(MouseY - MapY, MouseX - MapX);
            local PosX = 80 * cos(Ang);
            local PosY = 80 * sin(Ang);
            self:SetPoint("CENTER", PosX, PosY);
        end)
    end
end)

MinimapIcon:SetScript("OnDragStop", function(self)
    self:SetScript("OnUpdate", nil);
    local _, _, _, x, y = self:GetPoint(1);
    _Questic.SaveVariable("MinimapIconPositionx", x);
    _Questic.SaveVariable("MinimapIconPositiony", y);
end)

MinimapIcon:SetScript("OnMouseUp", function(self, button)
    local IsShown = false;
    if button == "RightButton" then
        _Questic.OptionsMenu.Call();
    else
        if self.DropDown then
            self.DropDown:Release();
            self.DropDown = nil;
        else
            local QuestTable = _Questic.GetQuestLogTable();
            local SelectItemList = {};
            for i, v in pairs(QuestTable) do
                if v.Type == "NQ" and v.Pushable == "1" then
                    local Item = {
                        Type = "DB_BL",
                        Text = v.Name,
                        Tooltip = v.Name,
                        Alignment = "LEFT",
                        Func = function()
                            _Questic.ShareQuest(nil, i)
                            self.DropDown:Release()
                            self.DropDown = nil
                        end
                    }
                    insert(SelectItemList, Item);
                end
            end
            insert(SelectItemList, {
                Type = "DB_BL",
                Text = "Close",
                Func = function()
                    self.DropDown:Release()
                    self.DropDown = nil
                end
            });
            self.DropDown = Widgets:CreateContainer(MinimapIcon);
            self.DropDown:PresetBorders()
            self.DropDown:Size(150, 200)
            self.DropDown:SetFrameStrata("HIGH")
            self.DropDown:EnableMouse(true)
            local Midx, Midy = UIParent:GetCenter();
            local Minix, Miniy = self:GetCenter();
            local Point, RefPoint = "", "";
            if Midy < Miniy then
                Point = "TOP"
                RefPoint = "BOTTOM"
            else
                Point = "BOTTOM"
                RefPoint = "TOP"
            end
            if Midx < Minix then
                Point = Point .. "RIGHT"
                RefPoint = RefPoint .. "LEFT"
            else
                Point = Point .. "LEFT"
                RefPoint = RefPoint .. "RIGHT"
            end
            self.DropDown:SetPoint(Point, self, RefPoint)
            local TotalHeight = self.DropDown:AddItems(SelectItemList)
            if TotalHeight and TotalHeight < 200 then
                self.DropDown:Size(nil, TotalHeight)
            end

            IsShown = true
        end
    end
    if IsShown then
        Tooltip:Hide();
    else
        Tooltip:SetTT(self,"Questic\n|cFFFFFFFFLeft-Click: Quick share\nRight-Click: Quick options",8,8);
    end
end)

MinimapIcon.Refresh = function(self)
    if _Questic.GetVariable("EnableMinimapIcon") then
        self:Show();
        self:SetPoint("CENTER", _Questic.GetVariable("MinimapIconPositionx"), _Questic.GetVariable("MinimapIconPositiony"));
        if _Questic.GetVariable("Enable") then
            self.Icon:SetDesaturated(false);
        else
            self.Icon:SetDesaturated(true);
        end
    else
        self:Hide();
    end
end

_Questic.MinimapIcon = MinimapIcon;

--[[
_Questic.MinimapIcon = {};



local function BuildQuickShareTable()
	local QuestTable = _Questic.GetQuestLogTable();
	local Table = {};
	for i,v in pairs(QuestTable) do
		if v.Type == "NQ" and v.Pushable == "1" then
			local Item = {Type = "DB_BL", Text = v.Name, Alignment = "LEFT",
			Func = function(self,UserInput,...) _Questic.ShareQuest(nil,i) end}

			table.insert(Table,Item);
		end
	end
	table.insert(Table,{Type = "DB_BL",Text = "Close", Func = function(self,UserInput,...) Questic_ADDON_OptionsDropDownMenu:SetOptions(nil,"QuesticQuickShareKey"); end});
	return Table;
end

local MinimapDropDownAnchorPoint = {Point = "TOPRIGHT",Ref = self,RefPoint = "CENTER",X = -8,Y = -8};

_Questic.CreateQuickShareMenu = function(Refresh)
	if Refresh and Questic_ADDON_OptionsDropDownMenu.Key ~= "QuesticQuickShareKey" then return nil end;
	local QuickShareTable = BuildQuickShareTable();
	local IsShown = false;
	if Refresh then
		IsShown = Questic_ADDON_OptionsDropDownMenu:RefreshOptions(QuickShareTable,"QuesticQuickShareKey",{Point = "TOPRIGHT",Ref = Questic_ADDON_MinimapIcon,RefPoint = "CENTER",X = -8,Y = -8});
	else
		IsShown = Questic_ADDON_OptionsDropDownMenu:SetOptions(QuickShareTable,"QuesticQuickShareKey",{Point = "TOPRIGHT",Ref = Questic_ADDON_MinimapIcon,RefPoint = "CENTER",X = -8,Y = -8});
	end
	return IsShown;
end
]]