local Widgets = _QuesticWidgets
local ipairs, wipe, CreateFrame, insert, type = ipairs, table.wipe, CreateFrame, table.insert, type;
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;

Widgets.CreateAdancedScrollFrame = function(self, Parent)
    local ScrollFrame = self:CreateFrame(Parent)

    ScrollFrame.ScrollFrame = self:CreateScrollFrame(ScrollFrame)

    ScrollFrame.Content = self:CreateFrame(ScrollFrame.ScrollFrame)
    ScrollFrame.Content:SetPoint("TOPLEFT")
    ScrollFrame.Content:Size(600, 100)
    ScrollFrame.Content:SetClampedToScreen(false)
    ScrollFrame.ScrollFrame:SetScrollChild(ScrollFrame.Content)

    ScrollFrame:SetScript("OnMouseWheel",
    function(_, delta)
        ScrollFrame:Refresh(-4 * delta, true)
    end)

    ScrollFrame.DownBtn = Widgets:CreateButton(ScrollFrame)
    ScrollFrame.DownBtn:SetPoint("BOTTOMRIGHT")
    ScrollFrame.DownBtn:Size(20, 20)
    ScrollFrame.DownBtn:SetIcon("Interface\\BUTTONS\\UI-MicroStream-Yellow.blp")
    ScrollFrame.DownBtn.OnClick = function(self)
        ScrollFrame:Refresh(20, true)
    end

    ScrollFrame.UpBtn = Widgets:CreateButton(ScrollFrame)
    ScrollFrame.UpBtn:SetPoint("TOPRIGHT", 0, ScrollFrame.TopOffset)
    ScrollFrame.UpBtn:Size(20, 20)
    ScrollFrame.UpBtn:SetIcon("Interface\\BUTTONS\\UI-MicroStream-Yellow.blp", 1, 0, 1, 0)
    ScrollFrame.UpBtn.OnClick = function(self)
        ScrollFrame:Refresh(-20, true)
    end

    ScrollFrame.SlidLine = Widgets:FetchTexture(ScrollFrame)
    ScrollFrame.SlidLine:SetPoint("TOP", ScrollFrame.UpBtn, "BOTTOM")
    ScrollFrame.SlidLine:SetPoint("BOTTOM", ScrollFrame.DownBtn, "TOP")
    ScrollFrame.SlidLine:SetWidth(2)
    ScrollFrame.SlidLine:SetColorTexture(1, 0.82, 0)

    ScrollFrame.SliderButton = Widgets:CreateFrame(ScrollFrame)
    ScrollFrame.SliderButton:Size(20, 20)
    ScrollFrame.SliderButton:SetPoint("TOP", ScrollFrame.SlidLine, "TOP", 0, -Padding)
    ScrollFrame.SliderButton:Backdrop()
    ScrollFrame.SliderButton:Backdrop("Backdrop", 1, 0.82, 0)

    ScrollFrame.ScrollFrameBackdrop = Widgets:CreateFrame(ScrollFrame)
    ScrollFrame.ScrollFrameBackdrop:Backdrop()
    ScrollFrame.ScrollFrameBackdrop:SetPoint("BOTTOMLEFT")
    ScrollFrame.ScrollFrameBackdrop:SetPoint("TOPRIGHT", ScrollFrame.UpBtn, "TOPLEFT", -Padding, 0)

    ScrollFrame.ScrollFrame:SetPoint("BOTTOMLEFT", ScrollFrame.ScrollFrameBackdrop, BorderSize, BorderSize)
    ScrollFrame.ScrollFrame:SetPoint("TOPRIGHT", ScrollFrame.ScrollFrameBackdrop, -BorderSize, -BorderSize)

    ScrollFrame.Refresh = function(self, ScrollOffset, Increment)
        ScrollOffset = ScrollOffset or self.ScrollOffset
        if Increment then 
            ScrollOffset = ScrollOffset + self.ScrollOffset
        else 
            ScrollOffset = ScrollOffset
        end 

        local ContentHeight = self.Content:GetHeight()
        if ScrollOffset < 0 then 
            ScrollOffset = 0
        end 
        if ContentHeight < self:GetHeight() - 2*BorderSize then 
            ScrollOffset = 0
        elseif ScrollOffset > ContentHeight - (self:GetHeight() - 2*BorderSize) then 
            ScrollOffset = ContentHeight - (self:GetHeight() - 2*BorderSize)
        end 

        self.ScrollFrame:SetVerticalScroll(ScrollOffset)
        local SliderHeight = self:GetHeight() - 2*Padding - self.UpBtn:GetHeight() - self.DownBtn:GetHeight()
        local SliderHeightMulti = (self:GetHeight() - 2*BorderSize) / ContentHeight
        if SliderHeightMulti > 1 then 
            SliderHeightMulti = 1
        end 

        self.SliderButton:SetPoint("TOP", ScrollFrame.SlidLine, "TOP", 0, -Padding - (SliderHeight * (ScrollOffset/self.Content:GetHeight())))
        self.SliderButton:Size(nil, SliderHeightMulti * SliderHeight)

        if ScrollOffset ~= self.ScrollOffset then 
            if self.OnScroll then 
                self.OnScroll(self, ScrollOffset, self.ScrollOffset)
            end 
        end 
        self.ScrollOffset = ScrollOffset
    end 

    ScrollFrame:SizeChanged(function()
        ScrollFrame.Content:Size(ScrollFrame:GetWidth() - 2 * BorderSize - Padding - ScrollFrame.UpBtn:GetWidth(), nil)
        ScrollFrame:Refresh()
    end)

    ScrollFrame.Content:SizeChanged(function()
        ScrollFrame.Content:SetWidth(ScrollFrame:GetWidth() - 2 * BorderSize - Padding - ScrollFrame.UpBtn:GetWidth())
        ScrollFrame:Refresh()
    end)

    ScrollFrame.ScrollOffset = 0;

    insert(ScrollFrame.LM, "CurrentScroll")
    insert(ScrollFrame.LM, "Content")
    insert(ScrollFrame.LM, "ScrollFrameBackdrop")
    insert(ScrollFrame.LM, "DownBtn")
    insert(ScrollFrame.LM, "UpBtn")
    insert(ScrollFrame.LM, "SlidLine")
    insert(ScrollFrame.LM, "SliderButton")
    insert(ScrollFrame.LM, "ScrollFrame")
    insert(ScrollFrame.LM, "ScrollOffset")
    insert(ScrollFrame.LM, "Refresh")

    return ScrollFrame
end 