local Widgets = _QuesticWidgets
local GetTime, ipairs, UIParent, CreateFrame, insert = GetTime, ipairs, UIParent, CreateFrame, table.insert;
local BorderSize = Widgets.BorderSize;

local Frames = {}

local function Release(self)
    self:Hide();
    self.OnFinished = nil;
    self.Released = true;
end

local function SetCooldown(self, Start, Duration)
    self:SetScript("OnUpdate", nil)
    if Duration == 0 then
        self:Hide();
        return nil
    end
    local Remaining = (GetTime() - Start) / Duration
    if Remaining >= 1 then
        self:Hide();
        return nil
    end
    if Remaining < 0.125 then
        if not self.CooldownBorder1 then
            self.CooldownBorder1 = Widgets:FetchTexture(self, "ARTWORK")
            self.CooldownBorder1:SetPoint("TOPRIGHT")
            self.CooldownBorder1:SetSize(30, BorderSize)
            self.CooldownBorder1:SetColorTexture(Widgets.HighlightColor.r, Widgets.HighlightColor.g, Widgets.HighlightColor.b, Widgets.HighlightColor.a)
        else
            self.CooldownBorder1:SetSize(30, BorderSize)
        end
    end
    if Remaining < 0.375 then
        if not self.CooldownBorder2 then
            self.CooldownBorder2 = Widgets:FetchTexture(self, "ARTWORK")
            self.CooldownBorder2:SetPoint("BOTTOMRIGHT")
            self.CooldownBorder2:SetSize(BorderSize, 60 - BorderSize)
            self.CooldownBorder2:SetColorTexture(Widgets.HighlightColor.r, Widgets.HighlightColor.g, Widgets.HighlightColor.b, Widgets.HighlightColor.a)
        else
            self.CooldownBorder2:SetSize(BorderSize, 60 - BorderSize)
        end
    end
    if Remaining < 0.625 then
        if not self.CooldownBorder3 then
            self.CooldownBorder3 = Widgets:FetchTexture(self, "ARTWORK")
            self.CooldownBorder3:SetPoint("BOTTOMLEFT")
            self.CooldownBorder3:SetSize(60 - BorderSize, BorderSize)
            self.CooldownBorder3:SetColorTexture(Widgets.HighlightColor.r, Widgets.HighlightColor.g, Widgets.HighlightColor.b, Widgets.HighlightColor.a)
        else
            self.CooldownBorder3:SetSize(60 - BorderSize, BorderSize)
        end
    end
    if Remaining < 0.875 then
        if not self.CooldownBorder4 then
            self.CooldownBorder4 = Widgets:FetchTexture(self, "ARTWORK")
            self.CooldownBorder4:SetPoint("TOPLEFT")
            self.CooldownBorder4:SetSize(BorderSize, 60 - BorderSize)
            self.CooldownBorder4:SetColorTexture(Widgets.HighlightColor.r, Widgets.HighlightColor.g, Widgets.HighlightColor.b, Widgets.HighlightColor.a)
        else
            self.CooldownBorder4:SetSize(BorderSize, 60 - BorderSize)
        end
    end
    if not self.CooldownBorder5 then
        self.CooldownBorder5 = Widgets:FetchTexture(self, "ARTWORK")
        self.CooldownBorder5:SetPoint("TOPRIGHT", self, "TOP")
        self.CooldownBorder5:SetSize(30 - BorderSize, BorderSize)
        self.CooldownBorder5:SetColorTexture(Widgets.HighlightColor.r, Widgets.HighlightColor.g, Widgets.HighlightColor.b, Widgets.HighlightColor.a)
    else
        self.CooldownBorder5:SetSize(30 - BorderSize, BorderSize)
    end
    self:Show();
    self:SetScript("OnUpdate", function()
        Remaining = (GetTime() - Start) / Duration
        if Remaining < 0.125 then
            self.CooldownBorder1:SetWidth(30 * (1 - Remaining / 0.125))
        elseif Remaining < 0.375 then
            if self.CooldownBorder1 then
                self.CooldownBorder1:Release();
                self.CooldownBorder1 = nil;
            end
            self.CooldownBorder2:SetHeight((60 - BorderSize) * (1 - (Remaining - 0.125) / 0.25))
        elseif Remaining < 0.625 then
            if self.CooldownBorder2 then
                self.CooldownBorder2:Release();
                self.CooldownBorder2 = nil;
            end
            self.CooldownBorder3:SetWidth((60 - BorderSize) * (1 - (Remaining - 0.375) / 0.25))
        elseif Remaining < 0.875 then
            if self.CooldownBorder3 then
                self.CooldownBorder3:Release();
                self.CooldownBorder3 = nil;
            end
            self.CooldownBorder4:SetHeight((60 - BorderSize) * (1 - (Remaining - 0.625) / 0.25))
        else
            if self.CooldownBorder4 then
                self.CooldownBorder4:Release();
                self.CooldownBorder4 = nil;
            end
            self.CooldownBorder5:SetWidth((30 - BorderSize) * (1 - (Remaining - 0.875) / 0.125))
        end
        if Remaining >= 1 then
            self:SetScript("OnUpdate", nil)
            self:Hide();
            if self.OnFinished then
                self.OnFinished();
                end
        end
    end)
end

Widgets.CreateCooldownFrame = function(self, Parent)
    local Frame = 0
    for i, v in ipairs(Frames) do
        if v.Released then
            Frame = i
            break
        end
    end
    if Frame == 0 then
        Frame = 1
        insert(Frames, Frame, CreateFrame("Frame", nil, Parent or UIParent))
        Frames[Frame]:SetClampedToScreen(true)

        Frames[Frame].Release = Release
        Frames[Frame].SetCooldown = SetCooldown

        Frames[Frame]:SetScript("OnHide", function(self)
            self:SetScript("OnUpdate", nil)
            if self.CooldownBorder1 then
                self.CooldownBorder1:Release()
                self.CooldownBorder1 = nil
            end
            if self.CooldownBorder2 then
                self.CooldownBorder2:Release()
                self.CooldownBorder2 = nil
            end
            if self.CooldownBorder3 then
                self.CooldownBorder3:Release()
                self.CooldownBorder3 = nil
            end
            if self.CooldownBorder4 then
                self.CooldownBorder4:Release()
                self.CooldownBorder4 = nil
            end
            if self.CooldownBorder5 then
                self.CooldownBorder5:Release()
                self.CooldownBorder5 = nil
            end
        end)
    end
    Frames[Frame]:SetParent(Parent or UIParent)
    Frames[Frame]:SetAllPoints()
    Frames[Frame].Released = false

    return Frames[Frame]
end