local Widgets = _QuesticWidgets
local type, ipairs, insert, UIParent, select = type, ipairs, table.insert, UIParent, select;

local function SetSelected(self, Value, UserInput)
    if type(Value) == "nil" then
        self.SelectDisplay:SetText("")
        return nil;
    end
    if self.SelectListFunc then
        self.SelectList = self.SelectListFunc()
    end
    if self.SelectList then
        local NewIndex
        if type(Value) == "number" and self.SelectList[Value] then
            NewIndex = Value
        elseif type(Value) == "string" then
            for i, v in ipairs(self.SelectList) do
                if v == Value then
                    NewIndex = i
                    break
                end
            end
        end
        if not NewIndex then
            return nil
        end
        self.SelectDisplay:SetText(self.SelectList[NewIndex])
        if self.OnSelect and UserInput then
            self.OnSelect(self, self.SelectList[NewIndex], NewIndex)
        end
    end
end

local function CallSelectList(self)
    if self.SelectListFunc then
        self.SelectList = self.SelectListFunc()
    end
    if self.SelectList then
        if self.SelectDropdownList then
            self.SelectDropdownList:Release()
            self.SelectDropdownList = nil
        else
            if Widgets.CreateContainer then
                local SelectItemList = {}
                for i, v in ipairs(self.SelectList) do
                    insert(SelectItemList,
                        {
                            Type = "DB_BL",
                            Text = v,
                            Alignment = self.SelectAlign,
                            Func = function()
                                self:SetSelected(i, true)
                                self:CallSelectList()
                            end
                        })
                end
                insert(SelectItemList,
                    {
                        VSpacing = 10,
                        Type = "DB_BL",
                        Text = "Cancel",
                        Func = function()
                            self:CallSelectList()
                        end
                    })
                self.SelectDropdownList = Widgets:CreateContainer(UIParent)
                --self.SelectDropdownList:PresetBorders("MouseOver", true)
                self.SelectDropdownList:PresetBorders()
                self.SelectDropdownList:Size(self:GetWidth(), 200)
                self.SelectDropdownList:SetFrameStrata("HIGH")
                self.SelectDropdownList:SetFrameLevel(7)
                self.SelectDropdownList:EnableMouse(true)
                if select(2, UIParent:GetCenter()) < select(2, self:GetCenter()) then
                    self.SelectDropdownList:SetPoint("TOPLEFT", self.SelectDisplay, "BOTTOMLEFT", 0, Widgets.BorderSize)
                    self.SelectDropdownList:SetPoint("TOPRIGHT", self.SelectDisplay, "BOTTOMRIGHT", 0, Widgets.BorderSize)
                else
                    self.SelectDropdownList:SetPoint("BOTTOMLEFT", self.SelectDisplay, "TOPLEFT", 0, -Widgets.BorderSize)
                    self.SelectDropdownList:SetPoint("BOTTOMRIGHT", self.SelectDisplay, "TOPRIGHT", 0, -Widgets.BorderSize)
                end
                local TotalHeight = self.SelectDropdownList:AddItems(SelectItemList)
                if TotalHeight and TotalHeight < 200 then
                    self.SelectDropdownList:Size(nil, TotalHeight)
                end
            end
        end
    end
end

Widgets.CreateSelectbutton = function(self, Parent, SelectAlign)
    local Selectbutton = Widgets:CreateFrame(Parent)

    Selectbutton.SelectAlign = SelectAlign

    Selectbutton.SelectDisplay = Widgets:CreateButton(Selectbutton)
    Selectbutton.SelectDisplay:Size(nil, 20)
    Selectbutton.SelectDisplay:SetPoint("BOTTOMLEFT")
    Selectbutton.SelectDisplay:SetPoint("BOTTOMRIGHT")
    Selectbutton.SelectDisplay.OnClick = function(self)
        if not Selectbutton.Disable then
            CallSelectList(Selectbutton)
        end
    end
    Selectbutton.SelectDisplay:SetText("", 11, 1, 1, 1, "LEFT","TOP")

    Selectbutton.SetSelected = SetSelected
    Selectbutton.CallSelectList = CallSelectList

    insert(Selectbutton.LM, "SelectDropdownList")
    insert(Selectbutton.LM, "SelectDisplay")
    insert(Selectbutton.LM, "SetSelected")
    insert(Selectbutton.LM, "OnSelect")
    insert(Selectbutton.LM, "SelectList")
    insert(Selectbutton.LM, "CallSelectList")
    insert(Selectbutton.LM, "SelectListFunc")

    return Selectbutton
end
