local Widgets = _QuesticWidgets
local gsub, strsplit, find, print, UIParent, ipairs, CreateFrame, type, insert = gsub, strsplit, string.find, print, UIParent, ipairs, CreateFrame, type, table.insert;
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;

local TextBoxes = {}

local function Release(self)
    self:Hide()
    self.Released = true
    self:SetAutoFocus(false)
    self:SetText("")
    self:SetMultiLine(false)
    self:SetNumeric(false)
    self:SetScript("OnEnter", nil)
    self:SetScript("OnLeave", nil)
    self:SetScript("OnEscapePressed", nil)
    self:SetScript("OnEnterPressed", nil)
    self:SetScript("OnEditFocusGained", nil)
    self:SetScript("OnEditFocusLost", nil)
    self:ClearFocus()
end

local function EditState(self, Start, Keep)
    if Start then
        self.EditBox:SetFocus()
    else
        if Keep then
            self:SetValue(self.EditBox:GetText(), true)
        end
        self.EditBox:ClearFocus()
        if self.EditBox.UpdateScript then
            self.EditBox.UpdateScript(self.EditBox:GetText())
        end 
    end
end

local function SetValue(self, Value, UserInput)
    if Value then
        if Value ~= self.Value then
            if self.NumericInputFilter then
                Value = gsub(Value, ",", ".")
                local FullNum, DecNum = strsplit(".", Value, 2)
                if FullNum or DecNum then
                    FullNum = gsub((FullNum or ""), "%D", "")
                    if FullNum == "" then
                        FullNum = "0"
                    end
                    DecNum = gsub((DecNum or ""), "%D", "")
                    if find(Value, "-") then
                        Value = "-"
                    else
                        Value = ""
                    end
                    if DecNum == "" or DecNum == "0" then
                        Value = Value .. FullNum
                    else
                        Value = Value .. FullNum .. "." .. DecNum
                    end
                else
                    print("Questic: Textbox input not a number.")
                    self.EditBox:SetText(self.Value)
                    return nil
                end
            end
            self.Value = Value
            self.EditBox:SetText(self.Value)
            if self.OnValueChanged and UserInput then
                self.OnValueChanged(self, self.Value)
            end
        end
    else
        self.EditBox:SetText(self.Value)
    end
end

Widgets.CreateBasicTextBox = function(self, Parent, MultiLine, HAlign, VAlign, UpdateScript)
    local TextBox = Widgets:CreateFrame(Parent or UIParent)
    TextBox:Backdrop()

    TextBox.EditBox = nil
    for i, v in ipairs(TextBoxes) do
        if v.Released then
            TextBox.EditBox = v
            TextBox.EditBox:SetParent(TextBox)
            TextBox.EditBox:Show()
        end
    end
    if not TextBox.EditBox then
        TextBox.EditBox = CreateFrame("EditBox", nil, TextBox)
        TextBox.EditBox:SetFont("Fonts\\FRIZQT__.TTF", 11)
        TextBox.EditBox:SetAutoFocus(false)

        TextBox.EditBox.Release = Release
    end
    TextBox.EditBox.Released = false
    TextBox.EditBox:SetMultiLine(MultiLine)
    TextBox.EditBox:SetJustifyH(HAlign or "CENTER")
    TextBox.EditBox:SetJustifyV(VAlign or "CENTER")

    --TextBox.EditBox:SetNumeric(Numeric) --Doesnt take , or . or - (only int values)

    TextBox.EditBox:SetScript("OnEnter",
        function(self)
            if not TextBox.Disable and (not TextBox.ActiveCheck or (TextBox.ActiveCheck and TextBox.ActiveCheck())) then
                TextBox.State = "MouseOver"
                TextBox:PresetBorders("MouseOver", self:HasFocus())
            end
        end)

    TextBox.EditBox:SetScript("OnLeave",
        function(self)
            TextBox.State = "Normal"
            TextBox:PresetBorders("Normal", self:HasFocus())
        end)

    TextBox.EditBox:SetScript("OnEscapePressed",
        function(self)
            TextBox:EditState()
        end)

    TextBox.EditBox:SetScript("OnEnterPressed",
        function(self)
            TextBox:EditState(false, true)
        end)

    TextBox.EditBox:SetScript("OnEditFocusGained",
        function(self)
            if not TextBox.Disable and (not TextBox.ActiveCheck or (TextBox.ActiveCheck and TextBox.ActiveCheck())) then
                self:HighlightText()
                TextBox:PresetBorders(TextBox.State, true)
                if TextBox.OnEditStart then
                    TextBox.OnEditStart()
                end
            else
                self:ClearFocus()
            end
        end)

    TextBox.EditBox:SetScript("OnEditFocusLost",
        function(self)
            TextBox:PresetBorders(TextBox.State, false)
            TextBox.EditBox:SetText(TextBox.Value)
            if TextBox.OnEditEnd then
                TextBox.OnEditEnd()
            end
        end)

        if UpdateScript then 
            TextBox.EditBox.UpdateScript = UpdateScript
            TextBox.EditBox:SetScript("OnTextChanged",
            function(self,UserInput)
                if UserInput then 
                    local LTC = 0
                    self:SetScript("OnUpdate",function(self,elapsed)
                        LTC = LTC + elapsed
                        if LTC > 0.6 then 
                            UpdateScript(self:GetText())
                            self:SetScript("OnUpdate",nil)
                        end 
                    end)
                end 
            end)
        else 
            TextBox.EditBox:SetScript("OnUpdate",nil)
            TextBox.EditBox:SetScript("OnTextChanged",nil)
        end 

    TextBox.EditBox:SetPoint("BOTTOMLEFT", BorderSize + Padding, BorderSize + Padding)
    TextBox.EditBox:SetPoint("TOPRIGHT", -(BorderSize + Padding), -(BorderSize + Padding))

    TextBox.SetValue = SetValue
    TextBox.EditState = EditState
    TextBox.Clear = function(self)
        self.Value = ""
        self:SetValue()
    end
    TextBox.HasFocus = function()
        return TextBox.EditBox:HasFocus()
    end
    TextBox.Value = ""

    insert(TextBox.LM, "EditBox");
    insert(TextBox.LM, "SetValue");
    insert(TextBox.LM, "OnValueChanged");
    insert(TextBox.LM, "HasFocus");
    insert(TextBox.LM, "EditState");
    insert(TextBox.LM, "Value");
    insert(TextBox.LM, "State");
    insert(TextBox.LM, "NumericInputFilter");
    insert(TextBox.LM, "ActiveCheck");
    insert(TextBox.LM, "OnEditEnd");
    insert(TextBox.LM, "OnEditStart");

    return TextBox
end

Widgets.CreateTextBox = function(self, Parent, MultiLine)
    local TextBox = Widgets:CreateFrame(Parent)

    TextBox.AcceptBtn = Widgets:CreateButton(TextBox)
    TextBox.AcceptBtn:SetPoint("BOTTOMLEFT")
    TextBox.AcceptBtn:SetPoint("BOTTOMRIGHT", TextBox, "BOTTOM", -Padding / 2, 0)
    TextBox.AcceptBtn:SetText("Accept")
    TextBox.AcceptBtn:Size(nil, 20)
    TextBox.AcceptBtn.OnClick = function(self)
        if not TextBox.Disable then
            TextBox.EditBox:EditState(false, true)
        end
    end
    TextBox.AcceptBtn:Hide()
    TextBox.AcceptBtn.ActiveCheck = function()
        return not TextBox.Disable
    end

    TextBox.EditCancelBtn = Widgets:CreateButton(TextBox)
    TextBox.EditCancelBtn:SetPoint("BOTTOMRIGHT")
    TextBox.EditCancelBtn:SetPoint("BOTTOMLEFT", TextBox, "BOTTOM", Padding / 2, 0)
    TextBox.EditCancelBtn:Size(nil, 20)
    TextBox.EditCancelBtn:SetText("Edit")
    TextBox.EditCancelBtn.OnClick = function(self)
        if not TextBox.Disable then
            TextBox.EditBox:EditState(not TextBox.EditBox:HasFocus())
        end
    end
    TextBox.EditCancelBtn.ActiveCheck = function()
        return not TextBox.Disable
    end

    TextBox.EditBox = Widgets:CreateBasicTextBox(TextBox, type(MultiLine) == "number", "LEFT")
    TextBox.EditBox:SetPoint("BOTTOMLEFT", TextBox.AcceptBtn, "TOPLEFT", 0, Padding)
    TextBox.EditBox:SetPoint("BOTTOMRIGHT", TextBox.EditCancelBtn, "TOPRIGHT", 0, Padding)
    TextBox.EditBox:Size(nil, 20 + (type(MultiLine) == "number" and (MultiLine - 1) * 11 or 0))
    TextBox.EditBox.OnValueChanged = function(_, Value)
        if not TextBox.Disable then
            TextBox:SetValue(Value, true)
        end
    end
    TextBox.EditBox.ActiveCheck = function()
        return not TextBox.Disable
    end

    TextBox.SetValue = function(self, Value, UserInput)
        Value = Value or self.Value
        self.EditBox:SetValue(Value)
        if self.Value ~= Value and UserInput then
            self.OnValueChanged(self, Value)
        end
    end

    TextBox.EditBox.OnEditStart = function()
        TextBox.AcceptBtn:Show()
        TextBox.EditCancelBtn:SetText("Cancel")
    end

    TextBox.EditBox.OnEditEnd = function()
        TextBox.AcceptBtn:Hide()
        TextBox.EditCancelBtn:SetText("Edit")
    end

    insert(TextBox.LM, "AcceptBtn");
    insert(TextBox.LM, "EditCancelBtn");
    insert(TextBox.LM, "EditBox");
    insert(TextBox.LM, "SetValue");
    insert(TextBox.LM, "Value");
    insert(TextBox.LM, "OnValueChanged");

    return TextBox
end
